//Const CallRate = 1 / 160
//Const RadPerDeg = 1.74532925199433E-02
#include <Adafruit_NeoPixel.h>


#define LED_PIN     D7
#define LED_COUNT  150

Adafruit_NeoPixel strip(LED_COUNT, LED_PIN, NEO_GRBW + NEO_KHZ800);

struct tpHSV{
  uint16_t H;
  byte S;
  byte V;
  byte W;
};

struct tpBlender{
  byte VectorH;
  byte VectorS;
  byte VectorV;
  byte VectorDeg;

  uint16_t MaxH;
  byte MaxS;
  byte MaxV;

  uint16_t MinH;
  byte MinS;
  byte MinV;
  
  byte HzH;
  byte HzS;
  byte HzV;
  byte HzT;
  
  unsigned long TimerH;
  unsigned long TimerS;
  unsigned long TimerV;
  unsigned long TimerT;
    
  byte Trail;
  bool Active;
  bool Wrap;
};

struct tpImage{
  tpHSV Matrix[15][10];
};

struct tpLayer{
  tpImage Image;
  tpBlender Blender[15][10];
};


tpLayer Layer[3];
unsigned long Timer=0;
int txtPosX, txtPosY;
int CurrentScene=0;
unsigned long ClockTimer;
unsigned long SceneTimer;
unsigned long ScrollTimer;
byte Brightness=255; //(max = 255)
byte XOBrightness=100;
float LastSEKUSD;
int ScrollPos;
String NewsText="";

#include "RemoteESP8266.h"
#include "Helpers.h"
#include "Snow.h"
#include "SirCliveMini.h"

int SceneTime=10000;
byte Scenes[6]={1,5,3,6,4,7};//{1,3,4};
byte SceneNr=0;
int ScrollSpeed=20;

void setup() {

  strip.begin();           // INITIALIZE NeoPixel strip object (REQUIRED)
  strip.show();            // Turn OFF all pixels ASAP
  strip.setBrightness(255);
  
  Serial.begin(115200);
  delay(100);

  ConnectToWifi();
  if (!noWifi) {
      SetupOTA();
  }
  
  XOBrightness=0;

  CurrentScene=0;

  ClockTimer=millis();
  SceneTimer=ClockTimer;
  
  LoadScene(Scenes[0],false);


}

void loop() {//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX MAIN LOOOP XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
    txtPosX=0;
    txtPosY=0;
    
    if (!noWifi) {
      ArduinoOTA.handle();
    }
  
    float Hz;
    Hz=1000.0/(float(millis())-float(Timer));
    Timer=millis();

    if(Timer-SceneTimer>SceneTime){
      if(XOBrightness==0){
        byte UB=(sizeof(Scenes)/sizeof(byte))-1;
        //Firebase.setInt(firebaseData, "LedPanel/UB", UB);
        byte NewScene;
        
        if(SceneNr+1>UB){
          SceneNr=0;
          NewScene=Scenes[SceneNr];
        }else{
          SceneNr=SceneNr+1;
          NewScene=Scenes[SceneNr];
        }
        SceneTimer=Timer;
        ClockTimer=Timer;
        
        //Firebase.setInt(firebaseData, "LedPanel/CurrentScene", NewScene);
        if(NewScene != CurrentScene){
          LoadScene(NewScene,false);
        }
      }else{
        if(XOBrightness>0){
          XOBrightness--;
          strip.setBrightness(XOBrightness);
        }
        
      }
    }else{
      if(XOBrightness<Brightness){
        XOBrightness++;
        strip.setBrightness(XOBrightness);
      }
    }


    
    if((CurrentScene==2)||(CurrentScene==4)){
      if(Timer-ClockTimer>50000){
        SetUpTime();
        ClearLayer(Layer[2]);

        String sTime=LatestTimeOnly.substring(0,1)+LatestTimeOnly.substring(1,2)+LatestTimeOnly.substring(3,4)+LatestTimeOnly.substring(4,5);
        int Width=TextWidth(sTime);
        txtPosX=(15-Width)/2;
        
        if(CurrentScene==2){
          TextToLayer(Layer[2],sTime,txtPosX,1,ToTpHSV(300,150,255,0));
          ApplyTwinkleAnimation(Layer[2], 10, 10, 30, 1, 1, 1, 50, 10, 5, 0, 0.9, 0.5);
        }else{
          TextToLayer(Layer[2],sTime,txtPosX,1,ToTpHSV(0,0,0,255));
        } 
        
        ClockTimer=Timer;
      }
    }

    if(CurrentScene==5 || CurrentScene==6 || CurrentScene==7){
      if(Timer-ScrollTimer>float(1000)/float(ScrollSpeed)){
        ScrollPos=ScrollPos-1;
        txtPosY=1;
        ClearLayer(Layer[2]);
        
        TextToLayer(Layer[2],NewsText,ScrollPos,txtPosY,ToTpHSV(0,0,0,255));//ändra till h>Scrollpos
        ScrollTimer=Timer;
      }

      int nWidth=-TextWidth(NewsText);
      if(ScrollPos-25>nWidth){//<sp-50    Keep Scene active as long as there is text to scroll
        SceneTimer=Timer;
      }else{
        SceneTimer=(Timer-SceneTime)-1;// Change Scene
        //XOBrightness=0;
        //strip.setBrightness(XOBrightness);
      }

    }

    Execute();

    //strip.fill(strip.Color(0,0,0,50),149-(Hz/10),149); // Speed Indicator
    strip.show();

}//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXxxxxxxxxxxxxxxxxxxxxxxxxXXXXXXX END LOOOP XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


void LoadScene(int Scene, bool BackgroundOnly){
  
  ClearAllLayers();

  bool BRcall = (Firebase.getInt(firebaseData, "LedPanel/Brightness"));
  Brightness = firebaseData.intData();
        
  if(Scene==0){
    if(noWifi){
      TextToLayer(Layer[2],"WIFI",0,0,ToTpHSV(21,240,255,0));
    }else{
      if(noFirebase){
        TextToLayer(Layer[2],"FIRE",0,0,ToTpHSV(21,240,255,0));
      }
    }
  }
  if(Scene==1){   //USD
    bool FBcall = (Firebase.getFloat(firebaseData, "DataDriller/SEKUSD"));
    float fSEKUSD=firebaseData.floatData();
        
    String sSEKUSD=String((round(fSEKUSD*100))/100);

    int Width=TextWidth(sSEKUSD);//S156
    txtPosX=(15-Width)/2;

    if(LastSEKUSD==fSEKUSD){
      TextToLayer(Layer[2],sSEKUSD,txtPosX,1,ToTpHSV(0,0,0,255));  

    }else{
      if(LastSEKUSD>fSEKUSD){
        TextToLayer(Layer[2],sSEKUSD,txtPosX,1,ToTpHSV(360,100,255,0)); 
      }
      if(LastSEKUSD<fSEKUSD){
        TextToLayer(Layer[2],sSEKUSD,txtPosX,1,ToTpHSV(135,100,255,0)); 
      }
    }
    if(LastSEKUSD==0){
      LastSEKUSD=fSEKUSD;
    }

    LayerFillH(Layer[0],215);
    LayerFillS(Layer[0],0);
    LayerFillV(Layer[0],0);

    ApplyTwinkleAnimation(Layer[0],0,0,60,0,0,2,0,0,100,0,0,1);

  }
    
  if(Scene==2){   //time only
    SetUpTime();
    String sTime=LatestTimeOnly.substring(0,1)+LatestTimeOnly.substring(1,2)+LatestTimeOnly.substring(3,4)+LatestTimeOnly.substring(4,5);
    
    int Width=TextWidth(sTime);//S156
    txtPosX=(15-Width)/2;

    TextToLayer(Layer[2],sTime,txtPosX,1,ToTpHSV(0,0,0,255));
  }


  if(Scene==3){// Weather /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    float fTemp;
    String sTemp;
    
    bool FBcall = (Firebase.getFloat(firebaseData, "LedPanel/S39/SSI"));
    if(FBcall==true) {
      fTemp = firebaseData.floatData();
      int iTemp=round(fTemp);
      sTemp=String(iTemp)+"'";
    }else{
      sTemp="?'";
    }
    
    delay(10);
    FBcall = (Firebase.getString(firebaseData, "LedPanel/S39/Weather"));
    String Weather =firebaseData.to<const char *>();//firebaseData.stringData();
    

    int wTemp=TextWidth(sTemp);
    txtPosX=(15-wTemp)/2;

    if(Weather=="Snow"){/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
      LayerFillH(Layer[0],217);
      LayerFillS(Layer[0],0);
      LoadPicToImage(Layer[0].Image,0,'V');

      ApplyTranslation (Layer[0],10, 270, 1, 0, 0, 0, true);
      if(!BackgroundOnly){TextToLayer(Layer[1],sTemp,0,0,ToTpHSV(217,167,255,0));}
    }
    

    if(Weather=="Rain"){/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
      LayerFillS(Layer[0], 166);
  
      Layer[0].Image.Matrix[0][0].V = 60;
      Layer[0].Image.Matrix[2][8].V = 80;
      Layer[0].Image.Matrix[4][3].V = 100;
      Layer[0].Image.Matrix[6][1].V = 120;
      Layer[0].Image.Matrix[8][6].V = 120;
      Layer[0].Image.Matrix[10][5].V = 100;
      Layer[0].Image.Matrix[12][7].V = 80;
      Layer[0].Image.Matrix[14][0].V = 60;
  
      Layer[0].Image.Matrix[0][0].H = 217;
      Layer[0].Image.Matrix[2][8].H = 217;
      Layer[0].Image.Matrix[4][3].H = 217;
      Layer[0].Image.Matrix[6][1].H = 217;
      Layer[0].Image.Matrix[8][6].H = 217;
      Layer[0].Image.Matrix[10][5].H = 217;
      Layer[0].Image.Matrix[12][7].H = 217;
      Layer[0].Image.Matrix[14][0].H = 217;
  
      ApplyTranslation(Layer[0], 20, 1, 1, 0, 0, 50, true);  
      if(!BackgroundOnly){TextToLayer(Layer[2],sTemp,txtPosX,1,ToTpHSV(0,0,0,255));}
    }

    if(Weather=="Cloudy"){////////////////////////////////////////////////////////////////////////////////////////////////////////////
      
      LayerFillH(Layer[0],215);
      LayerFillS(Layer[0],196);
      LayerFillV(Layer[0],100);
      
      tpHSV K;
      for(int X = 0;X<15;X++){
          for(int Y = 0;Y<5;Y++){
              K.H = 215;
              K.S = 196;
              K.V = (0.83 - ((4 - Y) * 0.05)) * 255;
              Layer[0].Image.Matrix[X][Y] = K;
          }
      }

      for(int x = 0;x<9;x++){
          Layer[1].Image.Matrix[x][6] = ToTpHSV(215, 196, 250,0);
          Layer[1].Image.Matrix[x][5] = ToTpHSV(215, 196, 250,0);
      }
      for(int x = 0;x<8;x++){
          Layer[1].Image.Matrix[x][4] = ToTpHSV(215, 196, 250,0);
      }
      
      LayerFillV(Layer[1],150);
      //LoadPicToImage(Layer[1].Image,3,'V');
      LoadPicToImage(Layer[1].Image,3,'s');
      LoadPicToImage(Layer[1].Image,4,'H');
    
      ApplyRollAnimation (Layer[0], 5, 7, 0, 1, 1, 1, 215, 206, 232, 200, 180, 120);
      ApplyTwinkleAnimation (Layer[1], 0, 0, 10, 0, 0, 1, 0, 0, 3, 0, 0, 0.5);
      if(!BackgroundOnly){TextToLayer(Layer[2],sTemp,txtPosX,1,ToTpHSV(0,0,0,255));}
    }
    
    if(Weather=="Partly cloudy"){////////////////////////////////////////////////////////////////////////////////////////////////////
      LayerFillS(Layer[0],230);
      LoadPicToImage(Layer[0].Image,5,'H');
      LoadPicToImage(Layer[0].Image,6,'v');
      LayerAdjustV(Layer[0],0.5);
      
      ApplyRollAnimation(Layer[0], 20, 0, 0,1, 0, 0, 49, 0, 255, 40, 200, 0);
      if(!BackgroundOnly){TextToLayer(Layer[2],sTemp,txtPosX,1,ToTpHSV(0,0,0,255));}
    }

    if(Weather=="Heavy rain"){////////////////////////////////////////////////////////////////////////////////////////////////////
      LayerFillS(Layer[0],100);//230
      
      LoadPicToImage(Layer[0].Image,7,'V');
      LoadPicToImage(Layer[0].Image,7,'H');
      //LayerAdjustV(Layer[0],0.5);
      
      ApplyTranslation(Layer[0], 20, 1, 1, 0, 0, 15, true);
      if(!BackgroundOnly){TextToLayer(Layer[2],sTemp,txtPosX,1,ToTpHSV(0,0,0,255));}
    }
  }

  if(Scene==4){// Time + Ball ////////////////////////////////////////////////////////////////////////////////////////////////////
    SetUpTime();
    LayerFillS(Layer[0],230);
    LoadPicToImage(Layer[0].Image,1,'H');
    LoadPicToImage(Layer[0].Image,2,'V');
    LayerAdjustV(Layer[0],0.8);
    
    ApplyRollAnimation (Layer[0], 30, 0, 100, 1, 0, 1, 360, 0, 140 , 10, 0, 0);
    
    String sTime=LatestTimeOnly.substring(0,1)+LatestTimeOnly.substring(1,2)+LatestTimeOnly.substring(3,4)+LatestTimeOnly.substring(4,5);
    
    int Width=TextWidth(sTime);//S156
    txtPosX=(15-Width)/2;
 
    //Firebase.setString(firebaseData,"LedPanel/substring",sTime); 
    TextToLayer(Layer[1],sTime,txtPosX,1,ToTpHSV(0,0,0,255));
  }

  if(Scene==5){// News1 ////////////////////////////////////////////////////////////////////////////////////////////////////
    NewsText=SetUpNews(1);
    LoadScene(3,true);
    ScrollPos=25;
    ScrollTimer=Timer;
    //XOBrightness=Brightness-1;
  }
  if(Scene==6){// News2 ////////////////////////////////////////////////////////////////////////////////////////////////////
    NewsText=SetUpNews(2);
    LoadScene(3,true);
    ScrollPos=25;
    ScrollTimer=Timer;
    //XOBrightness=Brightness-1;
  }
  if(Scene==7){// News3 ////////////////////////////////////////////////////////////////////////////////////////////////////
    NewsText=SetUpNews(3);
    LoadScene(3,true);
    ScrollPos=25;
    ScrollTimer=Timer;
    //XOBrightness=Brightness-1;
  }  
  
  CurrentScene=Scene;
 
}
